package main

import (
	"fmt"
	"io/fs"
	"os"
	"path/filepath"

	"github.com/osbuild/images/data/repositories"
	"github.com/osbuild/images/pkg/reporegistry"
	"github.com/osbuild/images/pkg/rpmmd"
)

// defaultDataDirs contains the default search paths to look for repository
// data. Note that the repositories are under a repositories/ sub-directory
// and contain a bunch of json files of the form "$distro_$version".json
// (but that is an implementation detail that the "images" library takes
// care of).
var defaultDataDirs = []string{
	"/etc/image-builder",
	"/usr/share/image-builder",
}

type repoConfig struct {
	DataDir    string
	ExtraRepos []string
}

func parseExtraRepo(extraRepo string) ([]rpmmd.RepoConfig, error) {
	// XXX: we want to eventually support more repos here:
	// - copr:@osbuild/osbuild (with full gpg retrival via the API)
	// - https://example.com/repo/fedora-42-x86_64/ (with sslverify=True)
	// But for now we only support local dirs
	stat, err := os.Stat(extraRepo)
	if err != nil {
		return nil, fmt.Errorf("cannot add extra repository: %v", err)
	}
	if !stat.IsDir() {
		return nil, fmt.Errorf("cannot add extra repository %v: must be a local directory", extraRepo)
	}
	extraRepo, err = filepath.Abs(extraRepo)
	if err != nil {
		return nil, err
	}

	checkGPG := false
	return []rpmmd.RepoConfig{
		{
			Id:           extraRepo,
			Name:         filepath.Base(extraRepo),
			BaseURLs:     []string{fmt.Sprintf("file:%s", extraRepo)},
			CheckGPG:     &checkGPG,
			CheckRepoGPG: &checkGPG,
		},
	}, nil
}

var newRepoRegistry = func(dataDir string, extraRepos []string) (*reporegistry.RepoRegistry, error) {
	var dataDirs []string
	if dataDir != "" {
		dataDirs = []string{dataDir}
	} else {
		dataDirs = defaultDataDirs
	}

	// XXX: think about sharing this with reporegistry?
	var fses []fs.FS
	for _, d := range dataDirs {
		fses = append(fses, os.DirFS(filepath.Join(d, "repositories")))
	}
	fses = append(fses, repos.FS)

	// XXX: should we support disabling the build-ins somehow?
	conf, err := reporegistry.LoadAllRepositoriesFromFS(fses)
	if err != nil {
		return nil, err
	}

	// XXX: this should probably go into manifestgen.Options as
	// a new Options.ExtraRepoConf eventually (just like OverrideRepos)
	for _, repo := range extraRepos {
		// XXX: this loads the extra repo unconditionally to all
		// distro/arch versions. we do not know in advance where
		// it belongs to
		extraRepo, err := parseExtraRepo(repo)
		if err != nil {
			return nil, err
		}
		for _, repoArchConfigs := range conf {
			for arch := range repoArchConfigs {
				archCfg := repoArchConfigs[arch]
				archCfg = append(archCfg, extraRepo...)
				repoArchConfigs[arch] = archCfg
			}
		}
	}

	return reporegistry.NewFromDistrosRepoConfigs(conf), nil
}
